""" Example application demonstrating job submission via bigjob with Amber11 
    
    
"""

import os
import time
import pdb
# BigJob implementation can be swapped here by importing another implementation,
# e.g. condor, cloud, azure
import sys
sys.path.insert(0, os.getcwd() + "/../")

COORDINATION_URL = "redis://ILikeBigJob_wITH-REdIS@gw68.quarry.iu.teragrid.org:6379"
RESOURCEMGR_URL  = "gram://gatekeeper.ranger.tacc.teragrid.org:2119/jobmanager-sge"

from bigjob import bigjob, subjob, description


### this is the number of sub-jobs you want to run
NUMBER_JOBS=8

def has_finished(state):
        state = state.lower()
        if state=="done" or state=="failed" or state=="canceled":
            return True
        else:
            return False


""" Test Job Submission via Advert """
if __name__ == "__main__":

    starttime=time.time()

    ##########################################################################################
    # make sure you are familiar with the queue structure on lonestar, your project id
    #        and the walltime limits on each queue. Change accordingly
    # Edit parameters for BigJob
    queue="normal" # you can switch to development for very short runs
    project="TG-XXXXXXXX"  # Shantenu's allocation. Please do not burn all SUs
    walltime=10  # There are limits on the time you can request. Read the user guide for the machine if unsure
    processes_per_node=16   # if you want more memory per process you can reduce this to 6, 2  or 1
    number_of_processes =32 # The total number of processes you want running, depends on the value above
    # to keep things clean: create the directory agent where you intend to run the script
    # you can start by doing this in $WORK and moving to $SCRATCH if you run out of space. This directory
    # will get big quickly so keep an eye out and do not put it in $HOME
    #workingdirectory= os.path.join(os.getcwd(), "agent") 
    # working directory for agent. Change this to the path to your agent directory on Ranger
    workingdirectory = "/path/on/ranger/agent"
    ##########################################################################################

    print "Start Pilot Job/BigJob at: " + RESOURCEMGR_URL
    bj = bigjob(COORDINATION_URL)
    bj.start_pilot_job( RESOURCEMGR_URL,
                        None,
                        number_of_processes,
                        queue,
                        project,
                        workingdirectory,
                        None,
                        walltime,
                        processes_per_node)

    print "Pilot Job/BigJob URL: " + bj.pilot_url + " State: " + str(bj.get_state())

    ##########################################################################################
    # Submit SubJob through BigJob
    # Here you can add any arguments to each replica, change the ouput and error filenames and so on
    # change this to your heart's content, but be careful
    jobs = []
    job_start_times = {}
    job_states = {}
    for i in range(0, NUMBER_JOBS):
        jd = description()
        jd.executable = "/scratch/projects/xsede/repex/amber11/bin/sander.MPI"
        jd.number_of_processes = "16"
        jd.spmd_variation = "mpi"
        jd.arguments = ["-O -i "+workingdirectory+"/inputfile -o outputfile -p "+workingdirectory+"/test.prmtop -c "+workingdirectory+"/test.prmcrd -r test.restrt"]
        jd.output = "sj-stdout-"+str(i)+".txt"
        jd.error = "sj-stderr-"+str(i)+".txt"

        sj = subjob()
        sj.submit_job(bj.pilot_url, jd)
        jobs.append(sj)
        job_start_times[sj]=time.time()
        job_states[sj] = sj.get_state()

    # busy wait for completion
    while 1:
        finish_counter=0
        result_map = {}
        for i in range(0, NUMBER_JOBS):
            old_state = job_states[jobs[i]]
            state = jobs[i].get_state()
            #print "Job " + str(jobs[i]) + " state: " + state
            if result_map.has_key(state)==False:
                result_map[state]=1
            else:
                result_map[state] = result_map[state]+1
            #pdb.set_trace()
            if old_state != state:
                print "Job " + str(jobs[i]) + " changed from: " + old_state + " to " + state
            if old_state != state and has_finished(state)==True:
                print "Job: " + str(jobs[i]) + " Runtime: " + str(time.time()-job_start_times[jobs[i]]) + " s."
            if has_finished(state)==True:
                finish_counter = finish_counter + 1
            job_states[jobs[i]]=state

        if finish_counter == NUMBER_JOBS:
            break
        time.sleep(2)

    runtime = time.time()-starttime
    print "Runtime: " + str(runtime) + " s; Runtime per Job: " + str(runtime/NUMBER_JOBS)
    ##########################################################################################
    # Cleanup - stop BigJob
    bj.cancel()
    #time.sleep(30)




import os
import time
import sys
import pdb

### COORDINATION_URL to point to advert server.  
COORDINATION_URL = "redis://ILikeBigJob_wITH-REdIS@gw68.quarry.iu.teragrid.org:6379"
from bigjob import bigjob, subjob, description

### EDIT based on the number of jobs you want to submit
NUMBER_JOBS=8

### BigJob main script#######
if __name__ == "__main__":
    starttime=time.time()

    ##########################################################################################
    # Edit parameters for BigJob
    queue = "small" # provide queue information
    project = "TG-XXXXXXXXXX" # provide allocation information 
    walltime = 10 # provide wall time in minutes
    size = 12 # provide total cores required
    #### Working directory should be in $SCRATCHDIR 
    workingdirectory = "/lustre/scratch/<username>/agent/"  
    userproxy = None # userproxy (not supported yet due to context issue w/ SAGA)
    #Provide resource url in the format sge-ssh://pmantha@target_machine_name
    lrms_url = "pbs-ssh://<username>@kraken-gsi.nics.teragrid.org"

    ##########################################################################################
    # Start Pilot Job/BigJob

    bj = bigjob(COORDINATION_URL)
    bj.start_pilot_job( lrms_url,
                        None,
                        size,
                        queue,
                        project,
                        workingdirectory,
                        userproxy,
                        walltime,
                        1)

    print "Pilot Job/BigJob URL: " + bj.pilot_url + " State: " + str(bj.get_state())

    ##########################################################################################
    ### Submit set of tasks T(A)
    ##########################################################################################
    for i in range(NUMBER_JOBS):
        jd = description()
        jd.executable = "/bin/echo"
        jd.number_of_processes = "4" 
        jd.spmd_variation = "mpi"
        jd.arguments = ["$INFRASTRUCTURE","$USERNAME"]
        jd.environment = ["INFRASTRUCTURE=kraken-mpi","USERNAME=test"]
        jd.output = "sj-stdout.txt"
        jd.error = "sj-stderr.txt"
        sj = subjob()
        sj.submit_job(bj.pilot_url, jd)

    ##########################################################################################
    ### Submit set of tasks T(B)
    ##########################################################################################
    for i in range(NUMBER_JOBS):
        jd = description()
        jd.executable = "/bin/echo"
        jd.number_of_processes = "1"
        jd.spmd_variation = "single"
        jd.arguments = ["$INFRASTRUCTURE","$USERNAME"]
        jd.environment = ["INFRASTRUCTURE=kraken-single","USERNAME=test"]
        jd.output = "sj-stdout.txt"
        jd.error = "sj-stderr.txt"
        sj = subjob()
        sj.submit_job(bj.pilot_url, jd)

    ########################################################################################
    # T(A) and T(B) execute parallely.....
    # wait for T(A) and T(B) to complete and then start T(C)
    bj.wait()
    runtime = time.time()-starttime
    print "Runtime: " + str(runtime) + " s; Runtime per Job: " + str(runtime/NUMBER_JOBS)

    ##########################################################################################
    ### Submit set of tasks T(C)
    ##########################################################################################
    starttime=time.time()
    for i in range(NUMBER_JOBS):
        jd = description()
        jd.executable = "/bin/date"
        jd.number_of_processes = "1"
        jd.spmd_variation = "single"
        jd.arguments = []
        jd.environment = []
        jd.output = "sj-stdout.txt"
        jd.error = "sj-stderr.txt"
        sj = subjob()
        sj.submit_job(bj.pilot_url, jd)

    ########################################################################################
    # wait for T(C) to complete
    bj.wait()
    runtime = time.time()-starttime
    print "Runtime: " + str(runtime) + " s; Runtime per Job: " + str(runtime/NUMBER_JOBS)
    ##########################################################################################

    ### The placement of bj.wait() provides dependencies between set of tasks implictly..

    # Cleanup - stop BigJob
    bj.cancel()



""" Example application demonstrating how to submit a N/2 
    /bin/echo jobs(SubJobs) each to two BigJobs. 
        
"""

import os
import time
import pdb
import sys
sys.path.insert(0, os.getcwd() + "/../")
COORDINATION_URL = "redis://ILikeBigJob_wITH-REdIS@gw68.quarry.iu.teragrid.org:6379"
from bigjob import bigjob, subjob, description


### This is the number of Jobs you want to run
NUMBER_JOBS=24
bjs=[]

if __name__ == "__main__":

    starttime=time.time()

    resource_list = []
    ## append machine parameters as below for each machine to be used.
    
    resource_list.append( {"lrms_url" : "sge-ssh://username@login#.ranger.tacc.utexas.edu",
                           "total_cores" : "32",
                           "tasks_per_node":"16",
                           "project" : "TG-XXXXXX",
                           "queue" : "normal",  
                           "workingdirectory": "/share/home/xxxxx/xxxxxx/agent",
                           "walltime": 10} )

    resource_list.append( {"lrms_url" : "sge-ssh://username@login#.ls4.tacc.utexas.edu",
                           "total_cores" : "24",
                           "tasks_per_node":"12",
                           "project" : "TG-MCB090174",
                           "queue" : "normal",  
                           "workingdirectory": "/home1/xxxxxx/xxxxxxx/agent",
                           "walltime": 10} )

    ##########################################################################################
    # Start bigjobs 
 
    for resource in resource_list:
        print "Start BigJob at: " + resource['lrms_url']
        bj=bigjob(COORDINATION_URL)
        bjs.append(bj)
        bj.start_pilot_job( resource['lrms_url'],
                             None,
                             resource['total_cores'],
                             resource['queue'],
                             resource['project'],
                             resource['workingdirectory'],
                             None,
                             resource['walltime'],
                             resource['tasks_per_node'])

    print "Pilot Job/BigJob URL: " + bj.pilot_url + " State: " + str(bj.get_state())

    ##########################################################################################
    # Submit Jobs through 1st BigJob
    # Here you can add any arguments to each Job, change the ouput and error filenames and so on
    # change this to your heart's content, but be careful
    for i in range(0, NUMBER_JOBS/2):
        jd = description()
        jd.executable = "/bin/echo"
        jd.number_of_processes = "4"
        jd.spmd_variation = "mpi" # for serial codes jd.spmd_variation="single"
        jd.arguments = ["BigJob", "$BIGJOB"]
        jd.environment = ["BIGJOB=1stBigJob"]
        jd.output = "sj-stdout-"+str(i)+".txt"
        jd.error = "sj-stderr-"+str(i)+".txt"

        sj = subjob()
        sj.submit_job(bjs[0].pilot_url, jd)  # bjs[0] represents the 1st bigjob

    ##########################################################################################

    # Submit Jobs through 2nd BigJob
    # Here you can add any arguments to each Job, change the ouput and error filenames and so on
    # change this to your heart's content, but be careful

    for i in range(0, NUMBER_JOBS/2):
        jd = description()
        jd.executable = "/bin/cat"
        jd.number_of_processes = "1"
        jd.spmd_variation = "single" 
        jd.arguments = ["/etc/hosts"]
        jd.environment =[]
        jd.output = "sj-stdout-"+str(i)+".txt"
        jd.error = "sj-stderr-"+str(i)+".txt"
        sj = subjob()
        sj.submit_job(bjs[1].pilot_url, jd) # bjs[1] represents the 2nd bigjob

    ##########################################################################################
    # waiting until all bigjobs complete
    ##########################################################################################

    for bj in bjs:
        print "waiting started..\n"
        bj.wait()
        print "waiting done..\n"

    runtime = time.time()-starttime
    print "Runtime: " + str(runtime) + " s; Runtime per Job: " + str(runtime/NUMBER_JOBS)

    ##########################################################################################
    # Cleanup - stop BigJob
    for bj in bjs:
        bj.cancel()



#!/usr/bin/env python

from bigjob import logger
from apiclient.discovery import build
from oauth2client.file import Storage
from oauth2client.client import OAuth2WebServerFlow
from oauth2client.tools import run
import httplib2
import os
import uuid
import time

import bliss.saga as saga

"""
AN OAUTH2 Client Id must be created at the Google API console at:

https://code.google.com/apis/console/

=> API Access

More information with respect to OAUTH: https://developers.google.com/compute/docs/api/how-tos/authorization
"""
OAUTH2_CLIENT_ID='1004462711324-55akehip32m59u6omdfrt9s8u8ehb0hm.apps.googleusercontent.com'
OAUTH2_CLIENT_SECRET='EIMML1W7anu0XijVghws0DY-'


GCE_PROJECT_ID='bigjob-pilot'

class gce_states:
    PROVISIONING="PROVISIONING"
    STAGING="STAGING"
    RUNNING="RUNNING"
    
    
class gcessh:
    """ Plugin for Google Compute Engine """

    def __init__(self, bootstrap_script=None):
        """Constructor"""
        self.bootstrap_script =  bootstrap_script
        self.id="bigjob-" + str(uuid.uuid1())
        self.network_ip=None
        
        # Do OAUTH authentication
        storage = Storage('gce.dat')
        self.credentials = storage.get()
        if self.credentials is None or self.credentials.invalid == True:
            flow = OAuth2WebServerFlow(
                                       client_id=OAUTH2_CLIENT_ID,
                                       client_secret=OAUTH2_CLIENT_SECRET,
                                       scope='https://www.googleapis.com/auth/compute',
                                       user_agent='bigjob-client/1.0')

            self.credentials = run(flow, storage)
        

    def run(self):
        request_dict = {
          "kind": "compute#instance",
          "disks": [],
          "networkInterfaces": [
            {
              "kind": "compute#instanceNetworkInterface",
              "accessConfigs": [
                {
                  "name": "External NAT",
                  "type": "ONE_TO_ONE_NAT"
                }
              ],
              "network": "https://www.googleapis.com/compute/v1beta12/projects/bigjob-pilot/networks/default"
            }
          ],         
          "zone": "https://www.googleapis.com/compute/v1beta12/projects/bigjob-pilot/zones/us-central1-a",
          "machineType": "https://www.googleapis.com/compute/v1beta12/projects/bigjob-pilot/machine-types/n1-standard-1",
          "name": self.id
        }
        
        http = httplib2.Http()
        http = self.credentials.authorize(http)
        gce = build("compute", "v1beta12", http=http)
        #result = gce.instances().get(instance="bigjob-pilot", project="bigjob-pilot").execute()
        gce.instances().insert(project=GCE_PROJECT_ID, body=request_dict).execute()
        
        #wait for compute instance to become active
        self.wait()
        
        # spawn BJ agent via SSH
        compute_instance_details = self.__get_instance_resource()
        logger.debug("Compute Instance Details: " + str(compute_instance_details))
        self.network_ip = compute_instance_details["networkInterfaces"][0]["accessConfigs"][0]['natIP']
        
        js = saga.job.Service(os.path.join("ssh://". self.network_ip))
        

    def wait(self):
        while self.get_state()!=gce_states.RUNNING:
            time.sleep(5)
        
    
    def get_state(self):
        result=self.__get_instance_resource()
        return result["status"]
    
    
    def cancel(self):
        http = httplib2.Http()
        http = self.credentials.authorize(http)
        gce = build("compute", "v1beta12", http=http)
        gce.instances().delete(project=GCE_PROJECT_ID, instance=self.id).execute()
        
    
    def __get_instance_resource(self):
        http = httplib2.Http()
        http = self.credentials.authorize(http)
        gce = build("compute", "v1beta12", http=http)
        result = gce.instances().get(project=GCE_PROJECT_ID, instance=self.id).execute()
        return result
 
 
     
if __name__ == "__main__":
    gce = gcessh()
    gce.run()
    print gce.get_state()



""" Example application demonstrating how to submit a simple 
    /bin/date with PilotJob. 
"""

import os
import time
import sys
import logging
from bigjob import logger
import pdb
logger.setLevel(logging.FATAL)
from pilot import PilotComputeService, ComputeDataService, State

### This is the number of jobs you want to run
NUMBER_JOBS=128

COORDINATION_URL = "advert://localhost/?dbtype=sqlite3"
# COORDINATION_URL = "redis://localhost:6379"

if __name__ == "__main__":

    start_time=time.time()
    pilot_compute_service = PilotComputeService(coordination_url=COORDINATION_URL)
    pilot_compute_description=[]

    pilot_compute_description.append({ "service_url": "ssh://localhost",
                                       "number_of_processes": 8,
                                       "working_directory": "/N/u/pmantha/agent",
                                     })
    pilot_compute_description.append({ "service_url": "pbs://localhost",
                                       "number_of_processes": 8,
                                       "working_directory": "/N/u/pmantha/agent",
                                       "queue":"batch",
                                       "processes_per_node":8,
                                       "walltime":60
                                     })
    pilot_compute_description.append({ "service_url": "pbs+ssh://sierra.futuregrid.org",
                                       "number_of_processes": 8,
                                       "working_directory": "/N/u/pmantha/agent",
                                       "queue":"batch",
                                       "processes_per_node":8,
                                       "walltime":60
                                     })
    pilot_compute_description.append({ "service_url": "pbs+ssh://alamo.futuregrid.org",
                                       "number_of_processes": 8,
                                       "working_directory": "/N/u/pmantha/agent",
                                       "queue":"batch",
                                       "processes_per_node":8,
                                       "walltime":60
                                     })
    pilot_compute_description.append({ "service_url": "pbs+ssh://hotel.futuregrid.org",
                                       "number_of_processes": 8,
                                       "working_directory": "/N/u/pmantha/agent",
                                       "queue":"batch",
                                       "processes_per_node":8,
                                       "walltime":60
                                     })
    pilot_compute_description.append({ "service_url": "pbs+ssh://india.futuregrid.org",
                                       "number_of_processes": 8,
                                       "working_directory": "/N/u/pmantha/agent",
                                       "queue":"batch",
                                       "processes_per_node":8,
                                       "walltime":60
                                     })

    for pcd in pilot_compute_description:
        pj = pilot_compute_service.create_pilot(pilot_compute_description=pcd)

    compute_data_service = ComputeDataService()
    compute_data_service.add_pilot_compute_service(pilot_compute_service)

    # submit work units
    for i in range(NUMBER_JOBS):
        compute_unit_description = {
                "executable": "/bin/date",
                "arguments": [""],
                "number_of_processes": 1,            
                "output": "stdout.txt",
                "error": "stderr.txt"
                }    
        compute_unit = compute_data_service.submit_compute_unit(compute_unit_description)

    logging.debug("Finished setup. Waiting for scheduling of compute units")
    compute_data_service.wait()

    logging.debug("Terminate Pilot Compute and Compute Data Service")
    compute_data_service.cancel()    
    pilot_compute_service.cancel()
    end_time=time.time()
    print "Total time to solution-" + str(round(end_time-start_time,2))



import os
import time
import sys

### COORDINATION_URL to point to advert server.  
COORDINATION_URL = "redis://ILikeBigJob_wITH-REdIS@gw68.quarry.iu.teragrid.org:6379"
from bigjob import bigjob, subjob, description

### EDIT based on the number of jobs you want to submit
NUMBER_JOBS=8

### BigJob main script#######
if __name__ == "__main__":
    starttime=time.time()

    ##########################################################################################
    # Edit parameters for BigJob
    queue = "workq" # provide queue information
    project = "loni_xxxxxx" # provide allocation information
    walltime = 10 # provide wall time in minutes
    processes_per_node = 4 # provide ppn
    required_processes = 8 # provide total processes required
    workingdirectory = "/home/username/agent" # provide working directory 
    userproxy = None # userproxy (not supported yet due to context issue w/ SAGA)
    lrms_url = "gram://username@poseidon1.loni.org/jobmanager-pbs" 

    ##########################################################################################
    # Start Pilot Job/BigJob

    bj = bigjob(COORDINATION_URL)
    bj.start_pilot_job( lrms_url,
                        None,
                        required_processes,
                        queue,
                        project,
                        workingdirectory,
                        userproxy,
                        walltime,
                        processes_per_node)

    print "Pilot Job/BigJob URL: " + bj.pilot_url + " State: " + str(bj.get_state())

    ##########################################################################################
    # Submit SubJob through BigJob
    for i in range(0, NUMBER_JOBS):
        jd = description()
        jd.executable = "/bin/echo"
        jd.number_of_processes = "4"
        jd.spmd_variation = "mpi" # single for serial jobs
        jd.environment=["NAME=user"]
        jd.arguments = ["hello","$NAME"]
        jd.output = "sj-stdout-"+str(i)+".txt"
        jd.error = "sj-stderr-"+str(i)+".txt"
        sj = subjob()
        sj.submit_job(bj.pilot_url, jd)

    ########################################################################################
    # wait for all jobs to complete
    bj.wait()

    runtime = time.time()-starttime
    print "Runtime: " + str(runtime) + " s; Runtime per Job: " + str(runtime/NUMBER_JOBS)


    ##########################################################################################
    # Cleanup - stop BigJob
    bj.cancel()
